@@ -118,7 +118,7 @@ require "rubygems/deprecate"
 # -The RubyGems Team
 
 module Gem
-  VERSION = '1.8.12'
+  VERSION = '1.8.15'
 
   ##
   # Raised when RubyGems is unable to load or activate a gem.  Contains the
@@ -639,10 +639,14 @@ module Gem
     index
   end
 
+  @yaml_loaded = false
+
   ##
   # Loads YAML, preferring Psych
 
   def self.load_yaml
+    return if @yaml_loaded
+
     begin
       gem 'psych', '~> 1.2', '>= 1.2.1' unless ENV['TEST_SYCK']
     rescue Gem::LoadError
@@ -661,6 +665,8 @@ module Gem
     # Now that we're sure some kind of yaml library is loaded, pull
     # in our hack to deal with Syck's DefaultKey ugliness.
     require 'rubygems/syck_hack'
+
+    @yaml_loaded = true
   end
 
   ##
@@ -980,9 +986,8 @@ module Gem
 
   def self.loaded_path? path
     # TODO: ruby needs a feature to let us query what's loaded in 1.8 and 1.9
-    $LOADED_FEATURES.find { |s|
-      s =~ /(^|\/)#{Regexp.escape path}#{Regexp.union(*Gem.suffixes)}$/
-    }
+    re = /(^|\/)#{Regexp.escape path}#{Regexp.union(*Gem.suffixes)}$/
+    $LOADED_FEATURES.any? { |s| s =~ re }
   end
 
   ##
