@@ -23,12 +23,12 @@ class TestGem < Gem::TestCase
   def assert_activate expected, *specs
     specs.each do |spec|
       case spec
-      when Array
-        Gem.activate(*spec)
-      when String
-        Gem.activate spec
+      when String then
+        Gem::Specification.find_by_name(spec).activate
+      when Gem::Specification then
+        spec.activate
       else
-        Gem.activate spec.name
+        flunk spec.inspect
       end
     end
 
@@ -51,17 +51,21 @@ class TestGem < Gem::TestCase
     Gem.unresolved_deps.values.map(&:to_s).sort
   end
 
+  # TODO: move these to specification
   def test_self_activate_via_require
-    new_spec "a", "1", "b" => "= 1"
-    new_spec "b", "1", nil, "lib/b/c.rb"
-    new_spec "b", "2", nil, "lib/b/c.rb"
+    a1 = new_spec "a", "1", "b" => "= 1"
+    b1 = new_spec "b", "1", nil, "lib/b/c.rb"
+    b2 = new_spec "b", "2", nil, "lib/b/c.rb"
 
-    Gem.activate "a", "= 1"
+    install_specs a1, b1, b2
+
+    a1.activate
     require "b/c"
 
     assert_equal %w(a-1 b-1), loaded_spec_names
   end
 
+  # TODO: move these to specification
   def test_self_activate_deep_unambiguous
     a1 = new_spec "a", "1", "b" => "= 1"
     b1 = new_spec "b", "1", "c" => "= 1"
@@ -71,7 +75,7 @@ class TestGem < Gem::TestCase
 
     install_specs a1, b1, b2, c1, c2
 
-    Gem.activate "a", "= 1"
+    a1.activate
     assert_equal %w(a-1 b-1 c-1), loaded_spec_names
   end
 
@@ -82,6 +86,7 @@ class TestGem < Gem::TestCase
     $LOADED_FEATURES.replace old_loaded_features
   end
 
+  # TODO: move these to specification
   def test_self_activate_ambiguous_direct
     save_loaded_features do
       a1 = new_spec "a", "1", "b" => "> 0"
@@ -90,9 +95,10 @@ class TestGem < Gem::TestCase
       c1 = new_spec "c", "1"
       c2 = new_spec "c", "2"
 
+      Gem::Specification.reset
       install_specs a1, b1, b2, c1, c2
 
-      Gem.activate "a", "= 1"
+      a1.activate
       assert_equal %w(a-1), loaded_spec_names
       assert_equal ["b (> 0)"], unresolved_names
 
@@ -103,6 +109,7 @@ class TestGem < Gem::TestCase
     end
   end
 
+  # TODO: move these to specification
   def test_self_activate_ambiguous_indirect
     save_loaded_features do
       a1 = new_spec "a", "1", "b" => "> 0"
@@ -113,7 +120,7 @@ class TestGem < Gem::TestCase
 
       install_specs a1, b1, b2, c1, c2
 
-      Gem.activate "a", "= 1"
+      a1.activate
       assert_equal %w(a-1), loaded_spec_names
       assert_equal ["b (> 0)"], unresolved_names
 
@@ -124,6 +131,7 @@ class TestGem < Gem::TestCase
     end
   end
 
+  # TODO: move these to specification
   def test_self_activate_ambiguous_unrelated
     save_loaded_features do
       a1 = new_spec "a", "1", "b" => "> 0"
@@ -135,7 +143,7 @@ class TestGem < Gem::TestCase
 
       install_specs a1, b1, b2, c1, c2, d1
 
-      Gem.activate "a", "= 1"
+      a1.activate
       assert_equal %w(a-1), loaded_spec_names
       assert_equal ["b (> 0)"], unresolved_names
 
@@ -146,6 +154,7 @@ class TestGem < Gem::TestCase
     end
   end
 
+  # TODO: move these to specification
   def test_self_activate_ambiguous_indirect_conflict
     save_loaded_features do
       a1 = new_spec "a", "1", "b" => "> 0"
@@ -157,7 +166,7 @@ class TestGem < Gem::TestCase
 
       install_specs a1, a2, b1, b2, c1, c2
 
-      Gem.activate "a", "= 2"
+      a2.activate
       assert_equal %w(a-2), loaded_spec_names
       assert_equal ["b (> 0)"], unresolved_names
 
@@ -168,13 +177,14 @@ class TestGem < Gem::TestCase
     end
   end
 
+  # TODO: move these to specification
   def test_require_already_activated
     save_loaded_features do
       a1 = new_spec "a", "1", nil, "lib/d.rb"
 
       install_specs a1 # , a2, b1, b2, c1, c2
 
-      Gem.activate "a", "= 1"
+      a1.activate
       assert_equal %w(a-1), loaded_spec_names
       assert_equal [], unresolved_names
 
@@ -185,6 +195,7 @@ class TestGem < Gem::TestCase
     end
   end
 
+  # TODO: move these to specification
   def test_require_already_activated_indirect_conflict
     save_loaded_features do
       a1 = new_spec "a", "1", "b" => "> 0"
@@ -196,8 +207,8 @@ class TestGem < Gem::TestCase
 
       install_specs a1, a2, b1, b2, c1, c2
 
-      Gem.activate "a", "= 1"
-      Gem.activate "c", "= 1"
+      a1.activate
+      c1.activate
       assert_equal %w(a-1 c-1), loaded_spec_names
       assert_equal ["b (> 0)"], unresolved_names
 
@@ -216,11 +227,26 @@ class TestGem < Gem::TestCase
     end
   end
 
-  def test_self_activate_loaded
-    util_spec 'foo', '1'
+  def test_require_does_not_glob
+    save_loaded_features do
+      a1 = new_spec "a", "1", nil, "lib/a1.rb"
 
-    assert Gem.activate 'foo'
-    refute Gem.activate 'foo'
+      install_specs a1
+
+      assert_raises ::LoadError do
+        require "a*"
+      end
+
+      assert_equal [], loaded_spec_names
+    end
+  end
+
+  # TODO: move these to specification
+  def test_self_activate_loaded
+    foo = util_spec 'foo', '1'
+
+    assert foo.activate
+    refute foo.activate
   end
 
   ##
@@ -243,15 +269,16 @@ class TestGem < Gem::TestCase
   #         [B] ~> 1.0
   #
   # and should resolve using b-1.0
+  # TODO: move these to specification
 
   def test_self_activate_over
-    util_spec 'a', '1.0', 'b' => '>= 1.0', 'c' => '= 1.0'
+    a = util_spec 'a', '1.0', 'b' => '>= 1.0', 'c' => '= 1.0'
     util_spec 'b', '1.0'
     util_spec 'b', '1.1'
     util_spec 'b', '2.0'
     util_spec 'c', '1.0', 'b' => '~> 1.0'
 
-    Gem.activate "a"
+    a.activate
 
     assert_equal %w[a-1.0 c-1.0], loaded_spec_names
     assert_equal ["b (>= 1.0, ~> 1.0)"], unresolved_names
@@ -324,6 +351,29 @@ class TestGem < Gem::TestCase
     end
   end
 
+  ##
+  # [A] depends on
+  #     [C]  = 1.0 depends on
+  #         [B] = 2.0
+  #     [B] ~> 1.0 (satisfied by 1.0)
+
+  def test_self_activate_checks_dependencies
+    a, _  = util_spec 'a', '1.0'
+            a.add_dependency 'c', '= 1.0'
+            a.add_dependency 'b', '~> 1.0'
+
+            util_spec 'b', '1.0'
+            util_spec 'b', '2.0'
+    c,  _ = util_spec 'c', '1.0', 'b' => '= 2.0'
+
+    e = assert_raises Gem::LoadError do
+      assert_activate nil, a, c, "b"
+    end
+
+    expected = "can't satisfy 'b (~> 1.0)', already activated 'b-2.0'"
+    assert_equal expected, e.message
+  end
+
   ##
   # [A] depends on
   #     [B] ~> 1.0 (satisfied by 1.0)
@@ -408,10 +458,20 @@ class TestGem < Gem::TestCase
 
   def test_self_available?
     util_make_gems
-    assert(Gem.available?("a"))
-    assert(Gem.available?("a", "1"))
-    assert(Gem.available?("a", ">1"))
-    assert(!Gem.available?("monkeys"))
+    Gem::Deprecate.skip_during do
+      assert(Gem.available?("a"))
+      assert(Gem.available?("a", "1"))
+      assert(Gem.available?("a", ">1"))
+      assert(!Gem.available?("monkeys"))
+    end
+  end
+
+  def test_self_bin_path_no_exec_name
+    e = assert_raises ArgumentError do
+      Gem.bin_path 'a'
+    end
+
+    assert_equal 'you must supply exec_name', e.message
   end
 
   def test_self_bin_path_bin_name
@@ -474,17 +534,12 @@ class TestGem < Gem::TestCase
   end
 
   def test_self_clear_paths
-    Gem.dir
-    Gem.path
-    searcher = Gem.searcher
-    source_index = Gem.source_index
+    assert_match(/gemhome$/, Gem.dir)
+    assert_match(/gemhome$/, Gem.path.first)
 
     Gem.clear_paths
 
-    assert_equal nil, Gem.instance_variable_get(:@gem_home)
-    assert_equal nil, Gem.instance_variable_get(:@gem_path)
-    refute_equal searcher, Gem.searcher
-    refute_equal source_index.object_id, Gem.source_index.object_id
+    assert_nil Gem::Specification.send(:class_variable_get, :@@all)
   end
 
   def test_self_configuration
@@ -507,8 +562,6 @@ class TestGem < Gem::TestCase
       install_gem foo
     end
 
-    Gem.source_index = nil
-
     gem 'foo'
 
     expected = File.join @gemhome, 'gems', foo.full_name, 'data', 'foo'
@@ -565,9 +618,23 @@ class TestGem < Gem::TestCase
 
     Gem.ensure_gem_subdirectories @gemhome
 
-    assert File.directory?(Gem.cache_dir(@gemhome))
+    assert File.directory? File.join(@gemhome, "cache")
   end
 
+  def test_self_ensure_gem_directories_safe_permissions
+    FileUtils.rm_r @gemhome
+    Gem.use_paths @gemhome
+
+    old_umask = File.umask
+    File.umask 0
+    Gem.ensure_gem_subdirectories @gemhome
+
+    assert_equal 0, File::Stat.new(@gemhome).mode & 022
+    assert_equal 0, File::Stat.new(File.join(@gemhome, "cache")).mode & 022
+  ensure
+    File.umask old_umask
+  end unless win_platform?
+
   def test_self_ensure_gem_directories_missing_parents
     gemdir = File.join @tempdir, 'a/b/c/gemdir'
     FileUtils.rm_rf File.join(@tempdir, 'a') rescue nil
@@ -577,7 +644,7 @@ class TestGem < Gem::TestCase
 
     Gem.ensure_gem_subdirectories gemdir
 
-    assert File.directory?(Gem.cache_dir(gemdir))
+    assert File.directory?(util_cache_dir)
   end
 
   unless win_platform? then # only for FS that support write protection
@@ -591,7 +658,7 @@ class TestGem < Gem::TestCase
 
       Gem.ensure_gem_subdirectories gemdir
 
-      refute File.exist?(Gem.cache_dir(gemdir))
+      refute File.exist?(util_cache_dir)
     ensure
       FileUtils.chmod 0600, gemdir
     end
@@ -608,7 +675,7 @@ class TestGem < Gem::TestCase
 
       Gem.ensure_gem_subdirectories gemdir
 
-      refute File.exist?(Gem.cache_dir(gemdir))
+      refute File.exist? File.join(gemdir, "gems")
     ensure
       FileUtils.chmod 0600, parent
     end
@@ -628,30 +695,26 @@ class TestGem < Gem::TestCase
   end
 
   def test_self_find_files
-    discover_path = File.join 'lib', 'sff', 'discover.rb'
     cwd = File.expand_path("test/rubygems", @@project_dir)
     $LOAD_PATH.unshift cwd
 
-    foo1 = quick_gem 'sff', '1' do |s|
-      s.files << discover_path
-    end
+    discover_path = File.join 'lib', 'sff', 'discover.rb'
 
-    foo2 = quick_gem 'sff', '2' do |s|
-      s.files << discover_path
-    end
+    foo1, foo2 = %w(1 2).map { |version|
+      spec = quick_gem 'sff', version do |s|
+        s.files << discover_path
+      end
 
-    path = File.join 'gems', foo1.full_name, discover_path
-    write_file(path) { |fp| fp.puts "# #{path}" }
+      write_file(File.join 'gems', spec.full_name, discover_path) do |fp|
+        fp.puts "# #{spec.full_name}"
+      end
 
-    path = File.join 'gems', foo2.full_name, discover_path
-    write_file(path) { |fp| fp.puts "# #{path}" }
-
-    @fetcher = Gem::FakeFetcher.new
-    Gem::RemoteFetcher.fetcher = @fetcher
-
-    Gem.source_index = util_setup_spec_fetcher foo1, foo2
+      spec
+    }
 
+    # HACK should be Gem.refresh
     Gem.searcher = nil
+    Gem::Specification.reset
 
     expected = [
       File.expand_path('test/rubygems/sff/discover.rb', @@project_dir),
@@ -668,9 +731,8 @@ class TestGem < Gem::TestCase
   def test_self_loaded_specs
     foo = quick_spec 'foo'
     install_gem foo
-    Gem.source_index = nil
 
-    Gem.activate 'foo'
+    foo.activate
 
     assert_equal true, Gem.loaded_specs.keys.include?('foo')
   end
@@ -687,15 +749,16 @@ class TestGem < Gem::TestCase
   def test_self_path_default
     util_path
 
-    if defined? APPLE_GEM_HOME
+    if defined?(APPLE_GEM_HOME)
       orig_APPLE_GEM_HOME = APPLE_GEM_HOME
       Object.send :remove_const, :APPLE_GEM_HOME
     end
-    Gem.instance_variable_set :@gem_path, nil
 
-    assert_equal [Gem.default_path, Gem.dir].flatten, Gem.path
+    Gem.instance_variable_set :@paths, nil
+
+    assert_equal [Gem.dir, *Gem.default_path].uniq, Gem.path
   ensure
-    Object.const_set :APPLE_GEM_HOME, orig_APPLE_GEM_HOME
+    Object.const_set :APPLE_GEM_HOME, orig_APPLE_GEM_HOME if orig_APPLE_GEM_HOME
   end
 
   unless win_platform?
@@ -704,11 +767,14 @@ class TestGem < Gem::TestCase
 
       Gem.clear_paths
       apple_gem_home = File.join @tempdir, 'apple_gem_home'
-      Gem.const_set :APPLE_GEM_HOME, apple_gem_home
+
+      old, $-w = $-w, nil
+      Object.const_set :APPLE_GEM_HOME, apple_gem_home
+      $-w = old
 
       assert_includes Gem.path, apple_gem_home
     ensure
-      Gem.send :remove_const, :APPLE_GEM_HOME
+      Object.send :remove_const, :APPLE_GEM_HOME
     end
 
     def test_self_path_APPLE_GEM_HOME_GEM_PATH
@@ -724,17 +790,15 @@ class TestGem < Gem::TestCase
   end
 
   def test_self_path_ENV_PATH
-    Gem.send :set_paths, nil
     path_count = Gem.path.size
     Gem.clear_paths
 
     ENV['GEM_PATH'] = @additional.join(File::PATH_SEPARATOR)
 
-    assert_equal @additional, Gem.path[0,2]
+    assert_equal [Gem.dir, *@additional], Gem.path
 
     assert_equal path_count + @additional.size, Gem.path.size,
                  "extra path components: #{Gem.path[2..-1].inspect}"
-    assert_equal Gem.dir, Gem.path.last
   end
 
   def test_self_path_duplicate
@@ -747,8 +811,7 @@ class TestGem < Gem::TestCase
 
     assert_equal @gemhome, Gem.dir
 
-    paths = [Gem.dir]
-    assert_equal @additional + paths, Gem.path
+    assert_equal [Gem.dir, *@additional], Gem.path
   end
 
   def test_self_path_overlap
@@ -760,8 +823,7 @@ class TestGem < Gem::TestCase
 
     assert_equal @gemhome, Gem.dir
 
-    paths = [Gem.dir]
-    assert_equal @additional + paths, Gem.path
+    assert_equal [Gem.dir, *@additional], Gem.path
   end
 
   def test_self_platforms
@@ -793,29 +855,20 @@ class TestGem < Gem::TestCase
   def test_self_refresh
     util_make_gems
 
-    a1_spec = File.join @gemhome, "specifications", @a1.spec_name
+    a1_spec = @a1.spec_file
+    moved_path = File.join @tempdir, File.basename(a1_spec)
 
-    FileUtils.mv a1_spec, @tempdir
-
-    refute Gem.source_index.gems.include?(@a1.full_name)
-
-    FileUtils.mv File.join(@tempdir, @a1.spec_name), a1_spec
+    FileUtils.mv a1_spec, moved_path
 
     Gem.refresh
 
-    assert_includes Gem.source_index.gems, @a1.full_name
-    assert_equal nil, Gem.instance_variable_get(:@searcher)
-  end
+    refute_includes Gem::Specification.all_names, @a1.full_name
 
-  def test_self_required_location
-    util_make_gems
+    FileUtils.mv moved_path, a1_spec
 
-    assert_equal File.join(@tempdir, *%w[gemhome gems c-1.2 lib code.rb]),
-                 Gem.required_location("c", "code.rb")
-    assert_equal File.join(@tempdir, *%w[gemhome gems a-1 lib code.rb]),
-                 Gem.required_location("a", "code.rb", "< 2")
-    assert_equal File.join(@tempdir, *%w[gemhome gems a-2 lib code.rb]),
-                 Gem.required_location("a", "code.rb", "= 2")
+    Gem.refresh
+
+    assert_includes Gem::Specification.all_names, @a1.full_name
   end
 
   def test_self_ruby_escaping_spaces_in_path
@@ -880,19 +933,20 @@ class TestGem < Gem::TestCase
     util_restore_RUBY_VERSION
   end
 
-  def test_self_searcher
-    assert_kind_of Gem::GemPathSearcher, Gem.searcher
-  end
-
-  def test_self_set_paths
+  def test_self_paths_eq
     other = File.join @tempdir, 'other'
     path = [@userhome, other].join File::PATH_SEPARATOR
-    Gem.send :set_paths, path
 
-    assert_equal [@userhome, other, @gemhome], Gem.path
+    #
+    # FIXME remove after fixing test_case
+    #
+    ENV["GEM_HOME"] = @gemhome
+    Gem.paths = { "GEM_PATH" => path }
+
+    assert_equal [@gemhome, @userhome, other], Gem.path
   end
 
-  def test_self_set_paths_nonexistent_home
+  def test_self_paths_eq_nonexistent_home
     ENV['GEM_HOME'] = @gemhome
     Gem.clear_paths
 
@@ -900,19 +954,37 @@ class TestGem < Gem::TestCase
 
     ENV['HOME'] = other
 
-    Gem.send :set_paths, other
+    Gem.paths = { "GEM_PATH" => other }
 
-    assert_equal [other, @gemhome], Gem.path
+    assert_equal [@gemhome, other], Gem.path
   end
 
   def test_self_source_index
-    assert_kind_of Gem::SourceIndex, Gem.source_index
+    Gem::Deprecate.skip_during do
+      assert_kind_of Gem::SourceIndex, Gem.source_index
+    end
   end
 
   def test_self_sources
     assert_equal %w[http://gems.example.com/], Gem.sources
   end
 
+  def test_self_try_activate_missing_dep
+    a = util_spec 'a', '1.0', 'b' => '>= 1.0'
+
+    a_file = File.join a.gem_dir, 'lib', 'a_file.rb'
+
+    write_file a_file do |io|
+      io.puts '# a_file.rb'
+    end
+
+    e = assert_raises Gem::LoadError do
+      Gem.try_activate 'a_file'
+    end
+
+    assert_match %r%Could not find b %, e.message
+  end
+
   def test_ssl_available_eh
     orig_Gem_ssl_available = Gem.ssl_available?
 
@@ -931,7 +1003,7 @@ class TestGem < Gem::TestCase
     Gem.use_paths @gemhome, @additional
 
     assert_equal @gemhome, Gem.dir
-    assert_equal @additional + [Gem.dir], Gem.path
+    assert_equal [Gem.dir, *@additional], Gem.path
   end
 
   def test_self_user_dir
@@ -947,23 +1019,9 @@ class TestGem < Gem::TestCase
     end
   end
 
-  def test_self_cache_dir
-    util_ensure_gem_dirs
-
-    assert_equal File.join(@gemhome, 'cache'), Gem.cache_dir
-    assert_equal File.join(@userhome, '.gem', Gem.ruby_engine, Gem::ConfigMap[:ruby_version], 'cache'), Gem.cache_dir(Gem.user_dir)
-  end
-
-  def test_self_cache_gem
-    util_ensure_gem_dirs
-
-    assert_equal File.join(@gemhome, 'cache', 'test.gem'), Gem.cache_gem('test.gem')
-    assert_equal File.join(@userhome, '.gem', Gem.ruby_engine, Gem::ConfigMap[:ruby_version], 'cache', 'test.gem'), Gem.cache_gem('test.gem', Gem.user_dir)
-  end
-
-  if Gem.win_platform? then
+  if Gem.win_platform? && '1.9' > RUBY_VERSION
+    # Ruby 1.9 properly handles ~ path expansion, so no need to run such tests.
     def test_self_user_home_userprofile
-      skip 'Ruby 1.9 properly handles ~ path expansion' unless '1.9' > RUBY_VERSION
 
       Gem.clear_paths
 
@@ -982,8 +1040,6 @@ class TestGem < Gem::TestCase
     end
 
     def test_self_user_home_user_drive_and_path
-      skip 'Ruby 1.9 properly handles ~ path expansion' unless '1.9' > RUBY_VERSION
-
       Gem.clear_paths
 
       # safe-keep env variables
@@ -1022,8 +1078,8 @@ class TestGem < Gem::TestCase
       install_gem foo
     end
 
-    Gem.source_index = nil
     Gem.searcher = nil
+    Gem::Specification.reset
 
     gem 'foo'
 
@@ -1049,6 +1105,23 @@ class TestGem < Gem::TestCase
     assert_equal :loaded, TEST_PLUGIN_EXCEPTION rescue nil
   end
 
+  def test_latest_load_paths
+    spec = quick_spec 'a', '4' do |s|
+      s.require_paths = ["lib"]
+    end
+
+    install_gem spec
+
+    # @exec_path = File.join spec.full_gem_path, spec.bindir, 'exec'
+    # @abin_path = File.join spec.full_gem_path, spec.bindir, 'abin'
+    # FileUtils.mkdir_p File.join(stem, "gems", "test-3")
+
+    Gem::Deprecate.skip_during do
+      expected = [File.join(@gemhome, "gems", "a-4", "lib")]
+      assert_equal expected, Gem.latest_load_paths
+    end
+  end
+
   def with_plugin(path)
     test_plugin_path = File.expand_path("test/rubygems/plugin/#{path}",
                                         @@project_dir)
@@ -1067,6 +1140,10 @@ class TestGem < Gem::TestCase
 
   def util_ensure_gem_dirs
     Gem.ensure_gem_subdirectories @gemhome
+
+    #
+    # FIXME what does this solve precisely? -ebh
+    #
     @additional.each do |dir|
       Gem.ensure_gem_subdirectories @gemhome
     end
@@ -1116,5 +1193,9 @@ class TestGem < Gem::TestCase
     Gem::Commands.send :remove_const, :InterruptCommand if
       Gem::Commands.const_defined? :InterruptCommand
   end
+
+  def util_cache_dir
+    File.join Gem.dir, "cache"
+  end
 end
 
